//
//  GLRule.h
//  GLMapKit
//
//  Created by Jeffrey Early on 12/30/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>

typedef enum {
	kGLUnfilterSelection = 0,
    kGLSelectedOnly = 1,
    kGLNotSelectedOnly = 2
} GLRuleSelectionFilterType;

typedef enum {
	kGLUnfilterRule = 0,
    kGLFilteredRule = 1,
    kGLElseFilteredRule = 2
} GLRuleFilterType;

@class GLSymbolizer;
@interface GLRule : NSObject <NSCoding, NSCopying> {
	NSString *name;
	NSString *title;
	NSString *abstract;
    NSNumber *minScaleDenominator;
	NSNumber *maxScaleDenominator;
    GLRuleSelectionFilterType selectionFilter;
    GLRuleFilterType filterType;
	NSPredicate *filter;
	NSMutableArray *symbolizers;
}

// The default rule is completely unfiltered, and will always display.


// If it's an else filter, nothing else matters
// 1. Must pass the scale test
// 2. Must pass the selection filter
// 3. Must pass filter test

/************************************************/
/*		Descriptions                            */
/************************************************/

#pragma mark -
#pragma mark Descriptions
#pragma mark

@property(copy) NSString *name, *title, *abstract;

/************************************************/
/*		Scale filter                            */
/************************************************/

#pragma mark -
#pragma mark Scale filter
#pragma mark

// If set to nil, these have no filtering effect.
@property(copy) NSNumber *minScaleDenominator, *maxScaleDenominator;

/************************************************/
/*		Selection filter                        */
/************************************************/

#pragma mark -
#pragma mark Selection filter
#pragma mark

// Filter based on the SpatialObject's selection status.
@property GLRuleSelectionFilterType selectionFilter;

/************************************************/
/*		Spatial object based filter             */
/************************************************/

#pragma mark -
#pragma mark Spatial object based filter
#pragma mark

// Filter based on a generic property of the SpatialObject.
@property GLRuleFilterType filterType;
@property(copy) NSPredicate *filter;

/************************************************/
/*		Symbolizers                             */
/************************************************/

#pragma mark -
#pragma mark Symbolizers
#pragma mark

@property(readonly, strong) NSMutableArray *symbolizers;

- (BOOL) addSymbolizer: (GLSymbolizer *) symbolizer;
- (BOOL) removeSymbolizer: (GLSymbolizer *) symbolizer;
- (BOOL) addSymbolizer: (GLSymbolizer *) symbolizer atIndex: (NSUInteger) index;
- (BOOL) moveSymbolizerFromIndex: (NSUInteger) oldIndex toIndex: (NSUInteger) newIndex;

/************************************************/
/*		Equality                                */
/************************************************/

#pragma mark -
#pragma mark Equality
#pragma mark

// Tests for complete equality, all properties and all symbolizers are equal
- (BOOL) isEqualToRule: (GLRule *) aRule;

// Only requires that the two rules share the same symbolizers in the same order
- (BOOL) isStructurallyEqualToRule: (GLRule *) aRule;



@end
